/*
 *  Licensed to the Apache Software Foundation (ASF) under one
 *  or more contributor license agreements.  See the NOTICE file
 *  distributed with this work for additional information
 *  regarding copyright ownership.  The ASF licenses this file
 *  to you under the Apache License, Version 2.0 (the
 *  "License"); you may not use this file except in compliance
 *  with the License.  You may obtain a copy of the License at
 *
 *        http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing,
 *  software distributed under the License is distributed on an
 *  "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 *  KIND, either express or implied.  See the License for the
 *  specific language governing permissions and limitations
 *  under the License.
 */
package org.apache.isis.viewer.bdd.common.fixtures.perform.checkthat;

import org.apache.isis.core.metamodel.consent.Consent;
import org.apache.isis.viewer.bdd.common.CellBinding;

/**
 * Parameterizes {@link ThatSubcommand}s.
 * 
 * @author Dan Haywood
 */
public enum AssertsValidity {

    VALID(true, "is valid for", "is valid", "valid"), INVALID(false, "is not valid for", "is not valid", "not valid", "invalid");

    private final String[] keys;
    private final boolean valid;

    AssertsValidity(final boolean valid, final String... keys) {
        this.keys = keys;
        this.valid = valid;
    }

    public String[] getKeys() {
        return keys;
    }

    public boolean isValid() {
        return valid;
    }

    public boolean satisfiedBy(final Consent validityConsent) {
        return validityConsent.isAllowed() && isValid() || validityConsent.isVetoed() && !isValid();
    }

    /**
     * The reason to use if the assertion is not valid.
     */
    public String getReason(final Consent validityConsent) {
        return isValid() ? validityConsent.getReason() : "(valid)";
    }

    /**
     * The binding whose current cell should be coloured if the assertion fails.
     */
    public CellBinding colorBinding(final CellBinding arg0Binding, final CellBinding thatBinding) {
        if (arg0Binding == null) {
            return thatBinding;
        }
        return isValid() ? arg0Binding : thatBinding;
    }

}